@@ -234,7 +234,10 @@ class AlignmentIterator(interfaces.AlignmentIterator):
             else:
                 assert len(value) == 1, (key, value)
                 value = value.pop()
-            alignment.annotations[AlignmentIterator.gf_mapping[key]] = value
+            try:
+                alignment.annotations[AlignmentIterator.gf_mapping[key]] = value
+            except KeyError:
+                pass
 
     @staticmethod
     def _store_per_column_annotations(alignment, gc, columns, skipped_columns):
@@ -251,7 +254,9 @@ class AlignmentIterator(interfaces.AlignmentIterator):
                     raise ValueError(
                         f"{key} length is {len(value)}, expected {columns}"
                     )
-                alignment.column_annotations[AlignmentIterator.gc_mapping[key]] = value
+                alignment.column_annotations[
+                    AlignmentIterator.gc_mapping.get(key, key)
+                ] = value
 
     @staticmethod
     def _store_per_sequence_annotations(alignment, gs):
@@ -267,7 +272,9 @@ class AlignmentIterator(interfaces.AlignmentIterator):
                 elif key == "DR":
                     record.dbxrefs = value
                 else:
-                    record.annotations[AlignmentIterator.gs_mapping[key]] = value
+                    record.annotations[
+                        AlignmentIterator.gs_mapping.get(key, key)
+                    ] = value
 
     @staticmethod
     def _store_per_sequence_and_per_column_annotations(alignment, gr):
@@ -277,9 +284,9 @@ class AlignmentIterator(interfaces.AlignmentIterator):
                     break
             else:
                 raise ValueError(f"Failed to find seqname {seqname}")
-            for keyword, letter_annotation in letter_annotations.items():
-                feature = AlignmentIterator.gr_mapping[keyword]
-                if keyword == "CSA":
+            for key, letter_annotation in letter_annotations.items():
+                feature = AlignmentIterator.gr_mapping.get(key, key)
+                if key == "CSA":
                     letter_annotation = letter_annotation.replace("-", "")
                 else:
                     letter_annotation = letter_annotation.replace(".", "")
@@ -533,7 +540,7 @@ class AlignmentWriter(interfaces.AlignmentWriter):
         for record in alignment.sequences:
             name = record.id.ljust(width)
             for key, value in record.annotations.items():
-                feature = self.gs_mapping[key]
+                feature = self.gs_mapping.get(key, key)
                 lines.append(f"#=GS {name}  {feature} {value}\n")
             if record.description:
                 lines.append(f"#=GS {name}  DE {record.description}\n")
@@ -558,8 +565,8 @@ class AlignmentWriter(interfaces.AlignmentWriter):
         #    alignment.column_annotations
         if alignment.column_annotations:
             for key, value in alignment.column_annotations.items():
-                feature = self.gc_mapping[key]
-                line = f"#=GC {feature}".ljust(start) + value + "\n"
+                feature = self.gc_mapping.get(key, key)
+                line = f"#=GC {feature} ".ljust(start) + value + "\n"
                 lines.append(line)
         lines.append("//\n")
         return "".join(lines)
@@ -592,7 +599,7 @@ class AlignmentWriter(interfaces.AlignmentWriter):
         indices.reverse()
         name = record.id.ljust(width)
         for key, value in record.letter_annotations.items():
-            feature = AlignmentWriter.gr_mapping[key]
+            feature = AlignmentWriter.gr_mapping.get(key, key)
             j = 0
             values = bytearray(b"." * len(aligned_sequence))
             for i, letter in enumerate(aligned_sequence):
@@ -600,7 +607,7 @@ class AlignmentWriter(interfaces.AlignmentWriter):
                     values[i] = ord(value[j])
                     j += 1
             value = values.decode()
-            line = f"#=GR {name}  {feature}".ljust(start) + value + "\n"
+            line = f"#=GR {name}  {feature} ".ljust(start) + value + "\n"
             yield line
 
 
