@@ -17,6 +17,7 @@ import itertools
 
 from ast import literal_eval
 from Bio.Phylo import BaseTree
+from Bio.Align import MultipleSeqAlignment
 
 
 class _BitString(str):
@@ -208,7 +209,7 @@ class _BitString(str):
         """Check if current bitstr1 is compatible with another bitstr2.
 
         Two conditions are considered as compatible:
-         1. bitstr1.contain(bitstr2) or vise versa;
+         1. bitstr1.contain(bitstr2) or vice versa;
          2. bitstr1.independent(bitstr2).
 
         """
@@ -539,7 +540,7 @@ def get_support(target_tree, trees, len_trees=None):
 
 
 def bootstrap(msa, times):
-    """Generate bootstrap replicates from a multiple sequence alignment object.
+    """Generate bootstrap replicates from a multiple sequence alignment (OBSOLETE).
 
     :Parameters:
         msa : MultipleSeqAlignment
@@ -562,11 +563,11 @@ def bootstrap(msa, times):
         yield item
 
 
-def bootstrap_trees(msa, times, tree_constructor):
+def bootstrap_trees(alignment, times, tree_constructor):
     """Generate bootstrap replicate trees from a multiple sequence alignment.
 
     :Parameters:
-        msa : MultipleSeqAlignment
+        alignment : Alignment or MultipleSeqAlignment object
             multiple sequence alignment to generate replicates.
         times : int
             number of bootstrap times.
@@ -574,17 +575,31 @@ def bootstrap_trees(msa, times, tree_constructor):
             tree constructor to be used to build trees.
 
     """
-    msas = bootstrap(msa, times)
-    for aln in msas:
-        tree = tree_constructor.build_tree(aln)
-        yield tree
+    if isinstance(alignment, MultipleSeqAlignment):
+        length = len(alignment[0])
+        for i in range(times):
+            bootstrapped_alignment = None
+            for j in range(length):
+                col = random.randint(0, length - 1)
+                if bootstrapped_alignment is None:
+                    bootstrapped_alignment = alignment[:, col : col + 1]
+                else:
+                    bootstrapped_alignment += alignment[:, col : col + 1]
+            tree = tree_constructor.build_tree(bootstrapped_alignment)
+            yield tree
+    else:
+        n, m = alignment.shape
+        for i in range(times):
+            cols = [random.randint(0, m - 1) for j in range(m)]
+            tree = tree_constructor.build_tree(alignment[:, cols])
+            yield tree
 
 
-def bootstrap_consensus(msa, times, tree_constructor, consensus):
+def bootstrap_consensus(alignment, times, tree_constructor, consensus):
     """Consensus tree of a series of bootstrap trees for a multiple sequence alignment.
 
     :Parameters:
-        msa : MultipleSeqAlignment
+        alignment : Alignment or MultipleSeqAlignment object
             Multiple sequence alignment to generate replicates.
         times : int
             Number of bootstrap times.
@@ -595,8 +610,8 @@ def bootstrap_consensus(msa, times, tree_constructor, consensus):
             ``majority_consensus``, ``adam_consensus``.
 
     """
-    trees = bootstrap_trees(msa, times, tree_constructor)
-    tree = consensus(list(trees))
+    trees = bootstrap_trees(alignment, times, tree_constructor)
+    tree = consensus(trees)
     return tree
 
 
