@@ -20,6 +20,8 @@ import sys
 
 from os.path import basename
 
+from typing import List
+
 from Bio.Seq import Seq
 from Bio.SeqRecord import SeqRecord
 
@@ -282,7 +284,7 @@ _INSTRUMENT_SPECIFIC_TAGS["abi_3530/3530xl"] = {
     "ScPa1": "The parameter string of size caller",
     "ScSt1": "Raw data start point. Set to 0 for 3500 data collection.",
     "SpeN1": "Active spectral calibration name",
-    "TrPa1": "Timming parameters",
+    "TrPa1": "Trimming parameters",
     "TrSc1": "Trace score.",
     "TrSc2": 'One of "Pass", "Fail", or "Check"',
     "phAR1": "Trace peak aria ratio",
@@ -325,7 +327,7 @@ _HEADFMT = ">H4sI2H3I"
 # directory data structure
 _DIRFMT = ">4sI2H4I"
 
-__global_tag_listing = []
+__global_tag_listing: List[str] = []
 for tag in _INSTRUMENT_SPECIFIC_TAGS.values():
     __global_tag_listing += tag.keys()
 
@@ -361,7 +363,7 @@ class AbiIterator(SequenceIterator):
             raise ValueError("Empty file.")
 
         if marker != b"ABIF":
-            raise OSError("File should start ABIF, not %r" % marker)
+            raise OSError(f"File should start ABIF, not {marker!r}")
         records = self.iterate(handle)
         return records
 
@@ -404,8 +406,8 @@ class AbiIterator(SequenceIterator):
                     annot[_EXTRACT[key]] = tag_data
 
         # set time annotations
-        annot["run_start"] = "%s %s" % (times["RUND1"], times["RUNT1"])
-        annot["run_finish"] = "%s %s" % (times["RUND2"], times["RUNT2"])
+        annot["run_start"] = f"{times['RUND1']} {times['RUNT1']}"
+        annot["run_finish"] = f"{times['RUND2']} {times['RUNT2']}"
 
         # raw data (for advanced end users benefit)
         annot["abif_raw"] = raw
