@@ -1,4 +1,5 @@
 # Copyright 2009-2020 by Peter Cock.  All rights reserved.
+# Copyright 2020 by Michael R. Crusoe
 #
 # This file is part of the Biopython distribution and governed by your
 # choice of the "Biopython License Agreement" or the "BSD 3-Clause License".
@@ -363,7 +364,6 @@ from math import log
 
 from Bio import BiopythonParserWarning
 from Bio import BiopythonWarning
-from Bio import BiopythonDeprecationWarning
 from Bio import StreamModeError
 from Bio.File import as_handle
 from Bio.Seq import Seq
@@ -373,7 +373,20 @@ from .Interfaces import _clean
 from .Interfaces import _get_seq_string
 from .Interfaces import SequenceIterator
 from .Interfaces import SequenceWriter
+from .Interfaces import _TextIOSource
 
+from typing import (
+    Any,
+    Callable,
+    Iterator,
+    IO,
+    List,
+    Mapping,
+    Optional,
+    Sequence,
+    Tuple,
+    Union,
+)
 
 # define score offsets. See discussion for differences between Sanger and
 # Solexa offsets.
@@ -381,7 +394,7 @@ SANGER_SCORE_OFFSET = 33
 SOLEXA_SCORE_OFFSET = 64
 
 
-def solexa_quality_from_phred(phred_quality):
+def solexa_quality_from_phred(phred_quality: float) -> float:
     """Convert a PHRED quality (range 0 to about 90) to a Solexa quality.
 
     PHRED and Solexa quality scores are both log transformations of a
@@ -475,7 +488,7 @@ def solexa_quality_from_phred(phred_quality):
         )
 
 
-def phred_quality_from_solexa(solexa_quality):
+def phred_quality_from_solexa(solexa_quality: float) -> float:
     """Convert a Solexa quality (which can be negative) to a PHRED quality.
 
     PHRED and Solexa quality scores are both log transformations of a
@@ -523,7 +536,7 @@ def phred_quality_from_solexa(solexa_quality):
     return 10 * log(10 ** (solexa_quality / 10.0) + 1, 10)
 
 
-def _get_phred_quality(record):
+def _get_phred_quality(record: SeqRecord) -> Union[List[float], List[int]]:
     """Extract PHRED qualities from a SeqRecord's letter_annotations (PRIVATE).
 
     If there are no PHRED qualities, but there are Solexa qualities, those are
@@ -547,7 +560,7 @@ def _get_phred_quality(record):
 
 # Only map 0 to 93, we need to give a warning on truncating at 93
 _phred_to_sanger_quality_str = {
-    qp: chr(min(126, qp + SANGER_SCORE_OFFSET)) for qp in range(0, 93 + 1)
+    qp: chr(min(126, qp + SANGER_SCORE_OFFSET)) for qp in range(93 + 1)
 }
 # Only map -5 to 93, we need to give a warning on truncating at 93
 _solexa_to_sanger_quality_str = {
@@ -556,7 +569,7 @@ _solexa_to_sanger_quality_str = {
 }
 
 
-def _get_sanger_quality_str(record):
+def _get_sanger_quality_str(record: SeqRecord) -> str:
     """Return a Sanger FASTQ encoded quality string (PRIVATE).
 
     >>> from Bio.Seq import Seq
@@ -673,7 +686,7 @@ def _get_sanger_quality_str(record):
 # Only map 0 to 62, we need to give a warning on truncating at 62
 assert 62 + SOLEXA_SCORE_OFFSET == 126
 _phred_to_illumina_quality_str = {
-    qp: chr(qp + SOLEXA_SCORE_OFFSET) for qp in range(0, 62 + 1)
+    qp: chr(qp + SOLEXA_SCORE_OFFSET) for qp in range(62 + 1)
 }
 # Only map -5 to 62, we need to give a warning on truncating at 62
 _solexa_to_illumina_quality_str = {
@@ -682,7 +695,7 @@ _solexa_to_illumina_quality_str = {
 }
 
 
-def _get_illumina_quality_str(record):
+def _get_illumina_quality_str(record: SeqRecord) -> str:
     """Return an Illumina 1.3 to 1.7 FASTQ encoded quality string (PRIVATE).
 
     Notice that due to the limited range of printable ASCII characters, a
@@ -753,11 +766,11 @@ _solexa_to_solexa_quality_str = {
 # Only map -5 to 62, we need to give a warning on truncating at 62
 _phred_to_solexa_quality_str = {
     qp: chr(min(126, int(round(solexa_quality_from_phred(qp))) + SOLEXA_SCORE_OFFSET))
-    for qp in range(0, 62 + 1)
+    for qp in range(62 + 1)
 }
 
 
-def _get_solexa_quality_str(record):
+def _get_solexa_quality_str(record: SeqRecord) -> str:
     """Return a Solexa FASTQ encoded quality string (PRIVATE).
 
     Notice that due to the limited range of printable ASCII characters, a
@@ -821,7 +834,7 @@ def _get_solexa_quality_str(record):
 
 
 # TODO - Default to nucleotide or even DNA?
-def FastqGeneralIterator(source):
+def FastqGeneralIterator(source: _TextIOSource) -> Iterator[Tuple[str, str, str]]:
     """Iterate over Fastq records as string tuples (not as SeqRecord objects).
 
     Arguments:
@@ -915,13 +928,9 @@ def FastqGeneralIterator(source):
     is that (provided there are no line breaks in the quality sequence) it
     would prevent the above problem with the "@" character.
     """
-    try:
-        handle = open(source)
-    except TypeError:
-        handle = source
+    with as_handle(source) as handle:
         if handle.read(0) != "":
             raise StreamModeError("Fastq files must be opened in text mode") from None
-    try:
         try:
             line = next(handle)
         except StopIteration:
@@ -987,27 +996,21 @@ def FastqGeneralIterator(source):
 
             if line is None:
                 break
-    finally:
-        if handle is not source:
-            handle.close()
 
 
-class FastqPhredIterator(SequenceIterator):
+class FastqPhredIterator(SequenceIterator[str]):
     """Parser for FASTQ files."""
 
-    def __init__(self, source, alphabet=None, title2ids=None):
+    def __init__(
+        self,
+        source: _TextIOSource,
+        alphabet: None = None,
+    ):
         """Iterate over FASTQ records as SeqRecord objects.
 
         Arguments:
          - source - input stream opened in text mode, or a path to a file
          - alphabet - optional alphabet, no longer used. Leave as None.
-         - title2ids (DEPRECATED) - A function that, when given the title line
-           from the FASTQ file (without the beginning >), will return the id,
-           name and description (in that order) for the record as a tuple of
-           strings.  If this is not given, then the entire title line will be
-           used as the description, and the first word as the id and name.
-
-        The use of title2ids matches that of Bio.SeqIO.FastaIO.
 
         For each sequence in a (Sanger style) FASTQ file there is a matching string
         encoding the PHRED qualities (integers between 0 and about 90) using ASCII
@@ -1058,9 +1061,9 @@ class FastqPhredIterator(SequenceIterator):
         >>> print(record.letter_annotations["phred_quality"])
         [26, 26, 26, 26, 26, 26, 26, 26, 26, 26, 26, 24, 26, 22, 26, 26, 13, 22, 26, 18, 24, 18, 18, 18, 18]
 
-        The title2ids argument is deprecated. Instead, please use a generator
-        function to modify the records returned by the parser. For example, to
-        store the mean PHRED quality in the record description, use
+        To modify the records returned by the parser, you can use a generator
+        function. For example, to store the mean PHRED quality in the record
+        description, use
 
         >>> from statistics import mean
         >>> def modify_records(records):
@@ -1079,36 +1082,15 @@ class FastqPhredIterator(SequenceIterator):
         """
         if alphabet is not None:
             raise ValueError("The alphabet argument is no longer supported")
-        if title2ids is not None:
-            warnings.warn(
-                "The title2ids argument is deprecated. Instead, please use a "
-                "generator function to modify records returned by the parser. "
-                "For example, to change the record description to a counter, "
-                "use\n"
-                "\n"
-                ">>> from statistics import mean\n"
-                ">>> def modify_records(records):\n"
-                "...     for record in records:\n"
-                "...         record.description = mean(record.letter_annotations['phred_quality'])\n"
-                "...         yield record\n"
-                "...\n"
-                ">>> with open('Quality/example.fastq') as handle:\n"
-                "...     for record in modify_records(FastqPhredIterator(handle)):\n"
-                "...         print(record.id, record.description)\n"
-                "\n",
-                BiopythonDeprecationWarning,
-            )
-        self.title2ids = title2ids
         super().__init__(source, mode="t", fmt="Fastq")
 
-    def parse(self, handle):
-        """Start parsing the file, and return a SeqRecord generator."""
+    def parse(self, handle: IO[str]) -> Iterator[SeqRecord]:
+        """Start parsing the file, and return a SeqRecord iterator."""
         records = self.iterate(handle)
         return records
 
-    def iterate(self, handle):
+    def iterate(self, handle: IO[str]) -> Iterator[SeqRecord]:
         """Parse the file and generate SeqRecord objects."""
-        title2ids = self.title2ids
         assert SANGER_SCORE_OFFSET == ord("!")
         # Originally, I used a list expression for each record:
         #
@@ -1121,15 +1103,12 @@ class FastqPhredIterator(SequenceIterator):
         }
 
         for title_line, seq_string, quality_string in FastqGeneralIterator(handle):
-            if title2ids:
-                id, name, descr = title2ids(title_line)
-            else:
-                descr = title_line
-                id = descr.split()[0]
-                name = id
+            descr = title_line
+            id = descr.split()[0]
+            name = id
             record = SeqRecord(Seq(seq_string), id=id, name=name, description=descr)
             try:
-                qualities = [q_mapping[letter] for letter in quality_string]
+                qualities = [q_mapping[letter2] for letter2 in quality_string]
             except KeyError:
                 raise ValueError("Invalid character in quality string") from None
             # For speed, will now use a dirty trick to speed up assigning the
@@ -1141,7 +1120,10 @@ class FastqPhredIterator(SequenceIterator):
             yield record
 
 
-def FastqSolexaIterator(source, alphabet=None, title2ids=None):
+def FastqSolexaIterator(
+    source: _TextIOSource,
+    alphabet: None = None,
+) -> Iterator[SeqRecord]:
     r"""Parse old Solexa/Illumina FASTQ like files (which differ in the quality mapping).
 
     The optional arguments are the same as those for the FastqPhredIterator.
@@ -1284,15 +1266,12 @@ def FastqSolexaIterator(source, alphabet=None, title2ids=None):
     }
 
     for title_line, seq_string, quality_string in FastqGeneralIterator(source):
-        if title2ids:
-            id, name, descr = title2ids(title_line)
-        else:
-            descr = title_line
-            id = descr.split()[0]
-            name = id
+        descr = title_line
+        id = descr.split()[0]
+        name = id
         record = SeqRecord(Seq(seq_string), id=id, name=name, description=descr)
         try:
-            qualities = [q_mapping[letter] for letter in quality_string]
+            qualities = [q_mapping[letter2] for letter2 in quality_string]
         # DO NOT convert these into PHRED qualities automatically!
         except KeyError:
             raise ValueError("Invalid character in quality string") from None
@@ -1302,7 +1281,10 @@ def FastqSolexaIterator(source, alphabet=None, title2ids=None):
         yield record
 
 
-def FastqIlluminaIterator(source, alphabet=None, title2ids=None):
+def FastqIlluminaIterator(
+    source: _TextIOSource,
+    alphabet: None = None,
+) -> Iterator[SeqRecord]:
     """Parse Illumina 1.3 to 1.7 FASTQ like files (which differ in the quality mapping).
 
     The optional arguments are the same as those for the FastqPhredIterator.
@@ -1340,15 +1322,12 @@ def FastqIlluminaIterator(source, alphabet=None, title2ids=None):
     }
 
     for title_line, seq_string, quality_string in FastqGeneralIterator(source):
-        if title2ids:
-            id, name, descr = title2ids(title_line)
-        else:
-            descr = title_line
-            id = descr.split()[0]
-            name = id
+        descr = title_line
+        id = descr.split()[0]
+        name = id
         record = SeqRecord(Seq(seq_string), id=id, name=name, description=descr)
         try:
-            qualities = [q_mapping[letter] for letter in quality_string]
+            qualities = [q_mapping[letter2] for letter2 in quality_string]
         except KeyError:
             raise ValueError("Invalid character in quality string") from None
         # Dirty trick to speed up this line:
@@ -1360,7 +1339,11 @@ def FastqIlluminaIterator(source, alphabet=None, title2ids=None):
 class QualPhredIterator(SequenceIterator):
     """Parser for QUAL files with PHRED quality scores but no sequence."""
 
-    def __init__(self, source, alphabet=None, title2ids=None):
+    def __init__(
+        self,
+        source: _TextIOSource,
+        alphabet: None = None,
+    ) -> None:
         """For QUAL files which include PHRED quality scores, but no sequence.
 
         For example, consider this short QUAL file::
@@ -1416,17 +1399,15 @@ class QualPhredIterator(SequenceIterator):
         """
         if alphabet is not None:
             raise ValueError("The alphabet argument is no longer supported")
-        self.title2ids = title2ids
         super().__init__(source, mode="t", fmt="QUAL")
 
-    def parse(self, handle):
-        """Start parsing the file, and return a SeqRecord generator."""
+    def parse(self, handle: IO) -> Iterator[SeqRecord]:
+        """Start parsing the file, and return a SeqRecord iterator."""
         records = self.iterate(handle)
         return records
 
-    def iterate(self, handle):
+    def iterate(self, handle: IO) -> Iterator[SeqRecord]:
         """Parse the file and generate SeqRecord objects."""
-        title2ids = self.title2ids
         # Skip any text before the first record (e.g. blank lines, comments)
         for line in handle:
             if line[0] == ">":
@@ -1439,14 +1420,11 @@ class QualPhredIterator(SequenceIterator):
                 raise ValueError(
                     "Records in Fasta files should start with '>' character"
                 )
-            if title2ids:
-                id, name, descr = title2ids(line[1:].rstrip())
-            else:
-                descr = line[1:].rstrip()
-                id = descr.split()[0]
-                name = id
+            descr = line[1:].rstrip()
+            id = descr.split()[0]
+            name = id
 
-            qualities = []
+            qualities: List[int] = []
             for line in handle:
                 if line[0] == ">":
                     break
@@ -1475,6 +1453,9 @@ class QualPhredIterator(SequenceIterator):
         raise ValueError("Unrecognised QUAL record format.")
 
 
+assert SANGER_SCORE_OFFSET == ord("!")
+
+
 class FastqPhredWriter(SequenceWriter):
     """Class to write standard FASTQ format files (using PHRED quality scores) (OBSOLETE).
 
@@ -1521,12 +1502,8 @@ class FastqPhredWriter(SequenceWriter):
     >>> os.remove("Quality/temp.fastq")
     """
 
-    assert SANGER_SCORE_OFFSET == ord("!")
-
-    def write_record(self, record):
+    def write_record(self, record: SeqRecord) -> None:
         """Write a single FASTQ record to the file."""
-        assert self._header_written
-        assert not self._footer_written
         self._record_written = True
         # TODO - Is an empty sequence allowed in FASTQ format?
         seq = record.seq
@@ -1541,20 +1518,20 @@ class FastqPhredWriter(SequenceWriter):
 
         # FASTQ files can include a description, just like FASTA files
         # (at least, this is what the NCBI Short Read Archive does)
-        id = self.clean(record.id)
+        id_ = self.clean(record.id) if record.id else ""
         description = self.clean(record.description)
-        if description and description.split(None, 1)[0] == id:
+        if description and description.split(None, 1)[0] == id_:
             # The description includes the id at the start
             title = description
         elif description:
-            title = f"{id} {description}"
+            title = f"{id_} {description}"
         else:
-            title = id
+            title = id_
 
         self.handle.write(f"@{title}\n{seq}\n+\n{qualities_str}\n")
 
 
-def as_fastq(record):
+def as_fastq(record: SeqRecord) -> str:
     """Turn a SeqRecord into a Sanger FASTQ formatted string.
 
     This is used internally by the SeqRecord's .format("fastq")
@@ -1568,14 +1545,14 @@ def as_fastq(record):
             "Record %s has sequence length %i but %i quality scores"
             % (record.id, len(seq_str), len(qualities_str))
         )
-    id = _clean(record.id)
+    id_ = _clean(record.id) if record.id else ""
     description = _clean(record.description)
-    if description and description.split(None, 1)[0] == id:
+    if description and description.split(None, 1)[0] == id_:
         title = description
     elif description:
-        title = f"{id} {description}"
+        title = f"{id_} {description}"
     else:
-        title = id
+        title = id_
     return f"@{title}\n{seq_str}\n+\n{qualities_str}\n"
 
 
@@ -1604,7 +1581,12 @@ class QualPhredWriter(SequenceWriter):
     >>> os.remove("Quality/temp.qual")
     """
 
-    def __init__(self, handle, wrap=60, record2title=None):
+    def __init__(
+        self,
+        handle: _TextIOSource,
+        wrap: int = 60,
+        record2title: Optional[Callable[[SeqRecord], str]] = None,
+    ) -> None:
         """Create a QUAL writer.
 
         Arguments:
@@ -1625,17 +1607,15 @@ class QualPhredWriter(SequenceWriter):
         """
         super().__init__(handle)
         # self.handle = handle
-        self.wrap = None
+        self.wrap: Optional[int] = None
         if wrap:
             if wrap < 1:
                 raise ValueError
-        self.wrap = wrap
+            self.wrap = wrap
         self.record2title = record2title
 
-    def write_record(self, record):
+    def write_record(self, record: SeqRecord) -> None:
         """Write a single QUAL record to the file."""
-        assert self._header_written
-        assert not self._footer_written
         self._record_written = True
 
         handle = self.handle
@@ -1644,15 +1624,15 @@ class QualPhredWriter(SequenceWriter):
         if self.record2title:
             title = self.clean(self.record2title(record))
         else:
-            id = self.clean(record.id)
+            id_ = self.clean(record.id) if record.id else ""
             description = self.clean(record.description)
-            if description and description.split(None, 1)[0] == id:
+            if description and description.split(None, 1)[0] == id_:
                 # The description includes the id at the start
                 title = description
             elif description:
                 title = f"{id} {description}"
             else:
-                title = id
+                title = id_
         handle.write(f">{title}\n")
 
         qualities = _get_phred_quality(record)
@@ -1666,7 +1646,7 @@ class QualPhredWriter(SequenceWriter):
             else:
                 raise
 
-        if wrap > 5:
+        if wrap is not None and wrap > 5:
             # Fast wrapping
             data = " ".join(qualities_strs)
             while True:
@@ -1692,20 +1672,20 @@ class QualPhredWriter(SequenceWriter):
             handle.write(data + "\n")
 
 
-def as_qual(record):
+def as_qual(record: SeqRecord) -> str:
     """Turn a SeqRecord into a QUAL formatted string.
 
     This is used internally by the SeqRecord's .format("qual")
     method and by the SeqIO.write(..., ..., "qual") function.
     """
-    id = _clean(record.id)
+    id_ = _clean(record.id) if record.id else ""
     description = _clean(record.description)
-    if description and description.split(None, 1)[0] == id:
+    if description and description.split(None, 1)[0] == id_:
         title = description
     elif description:
-        title = f"{id} {description}"
+        title = f"{id_} {description}"
     else:
-        title = id
+        title = id_
     lines = [f">{title}\n"]
 
     qualities = _get_phred_quality(record)
@@ -1778,10 +1758,8 @@ class FastqSolexaWriter(SequenceWriter):
     >>> os.remove("Quality/temp.fastq")
     """
 
-    def write_record(self, record):
+    def write_record(self, record: SeqRecord) -> None:
         """Write a single FASTQ record to the file."""
-        assert self._header_written
-        assert not self._footer_written
         self._record_written = True
 
         # TODO - Is an empty sequence allowed in FASTQ format?
@@ -1797,20 +1775,20 @@ class FastqSolexaWriter(SequenceWriter):
 
         # FASTQ files can include a description, just like FASTA files
         # (at least, this is what the NCBI Short Read Archive does)
-        id = self.clean(record.id)
+        id_ = self.clean(record.id) if record.id else ""
         description = self.clean(record.description)
-        if description and description.split(None, 1)[0] == id:
+        if description and description.split(None, 1)[0] == id_:
             # The description includes the id at the start
             title = description
         elif description:
-            title = f"{id} {description}"
+            title = f"{id_} {description}"
         else:
-            title = id
+            title = id_
 
         self.handle.write(f"@{title}\n{seq}\n+\n{qualities_str}\n")
 
 
-def as_fastq_solexa(record):
+def as_fastq_solexa(record: SeqRecord) -> str:
     """Turn a SeqRecord into a Solexa FASTQ formatted string.
 
     This is used internally by the SeqRecord's .format("fastq-solexa")
@@ -1823,15 +1801,15 @@ def as_fastq_solexa(record):
             "Record %s has sequence length %i but %i quality scores"
             % (record.id, len(seq_str), len(qualities_str))
         )
-    id = _clean(record.id)
+    id_ = _clean(record.id) if record.id else ""
     description = _clean(record.description)
-    if description and description.split(None, 1)[0] == id:
+    if description and description.split(None, 1)[0] == id_:
         # The description includes the id at the start
         title = description
     elif description:
-        title = f"{id} {description}"
+        title = f"{id_} {description}"
     else:
-        title = id
+        title = id_
     return f"@{title}\n{seq_str}\n+\n{qualities_str}\n"
 
 
@@ -1862,10 +1840,8 @@ class FastqIlluminaWriter(SequenceWriter):
     warning is issued.
     """
 
-    def write_record(self, record):
+    def write_record(self, record: SeqRecord) -> None:
         """Write a single FASTQ record to the file."""
-        assert self._header_written
-        assert not self._footer_written
         self._record_written = True
 
         # TODO - Is an empty sequence allowed in FASTQ format?
@@ -1881,20 +1857,20 @@ class FastqIlluminaWriter(SequenceWriter):
 
         # FASTQ files can include a description, just like FASTA files
         # (at least, this is what the NCBI Short Read Archive does)
-        id = self.clean(record.id)
+        id_ = self.clean(record.id) if record.id else ""
         description = self.clean(record.description)
-        if description and description.split(None, 1)[0] == id:
+        if description and description.split(None, 1)[0] == id_:
             # The description includes the id at the start
             title = description
         elif description:
-            title = f"{id} {description}"
+            title = f"{id_} {description}"
         else:
-            title = id
+            title = id_
 
         self.handle.write(f"@{title}\n{seq}\n+\n{qualities_str}\n")
 
 
-def as_fastq_illumina(record):
+def as_fastq_illumina(record: SeqRecord) -> str:
     """Turn a SeqRecord into an Illumina FASTQ formatted string.
 
     This is used internally by the SeqRecord's .format("fastq-illumina")
@@ -1907,18 +1883,22 @@ def as_fastq_illumina(record):
             "Record %s has sequence length %i but %i quality scores"
             % (record.id, len(seq_str), len(qualities_str))
         )
-    id = _clean(record.id)
+    id_ = _clean(record.id) if record.id else ""
     description = _clean(record.description)
-    if description and description.split(None, 1)[0] == id:
+    if description and description.split(None, 1)[0] == id_:
         title = description
     elif description:
-        title = f"{id} {description}"
+        title = f"{id_} {description}"
     else:
-        title = id
+        title = id_
     return f"@{title}\n{seq_str}\n+\n{qualities_str}\n"
 
 
-def PairedFastaQualIterator(fasta_source, qual_source, alphabet=None, title2ids=None):
+def PairedFastaQualIterator(
+    fasta_source: _TextIOSource,
+    qual_source: _TextIOSource,
+    alphabet: None = None,
+) -> Iterator[SeqRecord]:
     """Iterate over matched FASTA and QUAL files as SeqRecord objects.
 
     For example, consider this short QUAL file with PHRED quality scores::
@@ -1986,8 +1966,8 @@ def PairedFastaQualIterator(fasta_source, qual_source, alphabet=None, title2ids=
 
     from Bio.SeqIO.FastaIO import FastaIterator
 
-    fasta_iter = FastaIterator(fasta_source, title2ids=title2ids)
-    qual_iter = QualPhredIterator(qual_source, title2ids=title2ids)
+    fasta_iter = FastaIterator(fasta_source)
+    qual_iter = QualPhredIterator(qual_source)
 
     # Using zip wouldn't load everything into memory, but also would not catch
     # any extra records found in only one file.
@@ -2023,7 +2003,11 @@ def PairedFastaQualIterator(fasta_source, qual_source, alphabet=None, title2ids=
     # Done
 
 
-def _fastq_generic(in_file, out_file, mapping):
+def _fastq_generic(
+    in_file: _TextIOSource,
+    out_file: _TextIOSource,
+    mapping: Union[Sequence[str], Mapping[int, Optional[Union[str, int]]]],
+) -> int:
     """FASTQ helper function where can't have data loss by truncation (PRIVATE)."""
     # For real speed, don't even make SeqRecord and Seq objects!
     count = 0
@@ -2039,7 +2023,13 @@ def _fastq_generic(in_file, out_file, mapping):
     return count
 
 
-def _fastq_generic2(in_file, out_file, mapping, truncate_char, truncate_msg):
+def _fastq_generic2(
+    in_file: _TextIOSource,
+    out_file: _TextIOSource,
+    mapping: Union[Sequence[str], Mapping[int, Optional[Union[str, int]]]],
+    truncate_char: str,
+    truncate_msg: str,
+) -> int:
     """FASTQ helper function where there could be data loss by truncation (PRIVATE)."""
     # For real speed, don't even make SeqRecord and Seq objects!
     count = 0
@@ -2058,7 +2048,9 @@ def _fastq_generic2(in_file, out_file, mapping, truncate_char, truncate_msg):
     return count
 
 
-def _fastq_sanger_convert_fastq_sanger(in_file, out_file):
+def _fastq_sanger_convert_fastq_sanger(
+    in_file: _TextIOSource, out_file: _TextIOSource
+) -> int:
     """Fast Sanger FASTQ to Sanger FASTQ conversion (PRIVATE).
 
     Useful for removing line wrapping and the redundant second identifier
@@ -2069,7 +2061,7 @@ def _fastq_sanger_convert_fastq_sanger(in_file, out_file):
     """
     # Map unexpected chars to null
     mapping = "".join(
-        [chr(0) for ascii in range(0, 33)]
+        [chr(0) for ascii in range(33)]
         + [chr(ascii) for ascii in range(33, 127)]
         + [chr(0) for ascii in range(127, 256)]
     )
@@ -2077,7 +2069,9 @@ def _fastq_sanger_convert_fastq_sanger(in_file, out_file):
     return _fastq_generic(in_file, out_file, mapping)
 
 
-def _fastq_solexa_convert_fastq_solexa(in_file, out_file):
+def _fastq_solexa_convert_fastq_solexa(
+    in_file: _TextIOSource, out_file: _TextIOSource
+) -> int:
     """Fast Solexa FASTQ to Solexa FASTQ conversion (PRIVATE).
 
     Useful for removing line wrapping and the redundant second identifier
@@ -2087,7 +2081,7 @@ def _fastq_solexa_convert_fastq_solexa(in_file, out_file):
     """
     # Map unexpected chars to null
     mapping = "".join(
-        [chr(0) for ascii in range(0, 59)]
+        [chr(0) for ascii in range(59)]
         + [chr(ascii) for ascii in range(59, 127)]
         + [chr(0) for ascii in range(127, 256)]
     )
@@ -2095,7 +2089,9 @@ def _fastq_solexa_convert_fastq_solexa(in_file, out_file):
     return _fastq_generic(in_file, out_file, mapping)
 
 
-def _fastq_illumina_convert_fastq_illumina(in_file, out_file):
+def _fastq_illumina_convert_fastq_illumina(
+    in_file: _TextIOSource, out_file: _TextIOSource
+) -> int:
     """Fast Illumina 1.3+ FASTQ to Illumina 1.3+ FASTQ conversion (PRIVATE).
 
     Useful for removing line wrapping and the redundant second identifier
@@ -2105,7 +2101,7 @@ def _fastq_illumina_convert_fastq_illumina(in_file, out_file):
     """
     # Map unexpected chars to null
     mapping = "".join(
-        [chr(0) for ascii in range(0, 64)]
+        [chr(0) for ascii in range(64)]
         + [chr(ascii) for ascii in range(64, 127)]
         + [chr(0) for ascii in range(127, 256)]
     )
@@ -2113,7 +2109,9 @@ def _fastq_illumina_convert_fastq_illumina(in_file, out_file):
     return _fastq_generic(in_file, out_file, mapping)
 
 
-def _fastq_illumina_convert_fastq_sanger(in_file, out_file):
+def _fastq_illumina_convert_fastq_sanger(
+    in_file: _TextIOSource, out_file: _TextIOSource
+) -> int:
     """Fast Illumina 1.3+ FASTQ to Sanger FASTQ conversion (PRIVATE).
 
     Avoids creating SeqRecord and Seq objects in order to speed up this
@@ -2121,15 +2119,17 @@ def _fastq_illumina_convert_fastq_sanger(in_file, out_file):
     """
     # Map unexpected chars to null
     mapping = "".join(
-        [chr(0) for ascii in range(0, 64)]
-        + [chr(33 + q) for q in range(0, 62 + 1)]
+        [chr(0) for ascii in range(64)]
+        + [chr(33 + q) for q in range(62 + 1)]
         + [chr(0) for ascii in range(127, 256)]
     )
     assert len(mapping) == 256
     return _fastq_generic(in_file, out_file, mapping)
 
 
-def _fastq_sanger_convert_fastq_illumina(in_file, out_file):
+def _fastq_sanger_convert_fastq_illumina(
+    in_file: _TextIOSource, out_file: _TextIOSource
+) -> int:
     """Fast Sanger FASTQ to Illumina 1.3+ FASTQ conversion (PRIVATE).
 
     Avoids creating SeqRecord and Seq objects in order to speed up this
@@ -2139,8 +2139,8 @@ def _fastq_sanger_convert_fastq_illumina(in_file, out_file):
     # Map unexpected chars to null
     trunc_char = chr(1)
     mapping = "".join(
-        [chr(0) for ascii in range(0, 33)]
-        + [chr(64 + q) for q in range(0, 62 + 1)]
+        [chr(0) for ascii in range(33)]
+        + [chr(64 + q) for q in range(62 + 1)]
         + [trunc_char for ascii in range(96, 127)]
         + [chr(0) for ascii in range(127, 256)]
     )
@@ -2154,7 +2154,9 @@ def _fastq_sanger_convert_fastq_illumina(in_file, out_file):
     )
 
 
-def _fastq_solexa_convert_fastq_sanger(in_file, out_file):
+def _fastq_solexa_convert_fastq_sanger(
+    in_file: _TextIOSource, out_file: _TextIOSource
+) -> int:
     """Fast Solexa FASTQ to Sanger FASTQ conversion (PRIVATE).
 
     Avoids creating SeqRecord and Seq objects in order to speed up this
@@ -2162,7 +2164,7 @@ def _fastq_solexa_convert_fastq_sanger(in_file, out_file):
     """
     # Map unexpected chars to null
     mapping = "".join(
-        [chr(0) for ascii in range(0, 59)]
+        [chr(0) for ascii in range(59)]
         + [
             chr(33 + int(round(phred_quality_from_solexa(q))))
             for q in range(-5, 62 + 1)
@@ -2173,7 +2175,9 @@ def _fastq_solexa_convert_fastq_sanger(in_file, out_file):
     return _fastq_generic(in_file, out_file, mapping)
 
 
-def _fastq_sanger_convert_fastq_solexa(in_file, out_file):
+def _fastq_sanger_convert_fastq_solexa(
+    in_file: _TextIOSource, out_file: _TextIOSource
+) -> int:
     """Fast Sanger FASTQ to Solexa FASTQ conversion (PRIVATE).
 
     Avoids creating SeqRecord and Seq objects in order to speed up this
@@ -2183,8 +2187,8 @@ def _fastq_sanger_convert_fastq_solexa(in_file, out_file):
     # Map unexpected chars to null
     trunc_char = chr(1)
     mapping = "".join(
-        [chr(0) for ascii in range(0, 33)]
-        + [chr(64 + int(round(solexa_quality_from_phred(q)))) for q in range(0, 62 + 1)]
+        [chr(0) for ascii in range(33)]
+        + [chr(64 + int(round(solexa_quality_from_phred(q)))) for q in range(62 + 1)]
         + [trunc_char for ascii in range(96, 127)]
         + [chr(0) for ascii in range(127, 256)]
     )
@@ -2198,7 +2202,9 @@ def _fastq_sanger_convert_fastq_solexa(in_file, out_file):
     )
 
 
-def _fastq_solexa_convert_fastq_illumina(in_file, out_file):
+def _fastq_solexa_convert_fastq_illumina(
+    in_file: _TextIOSource, out_file: _TextIOSource
+) -> int:
     """Fast Solexa FASTQ to Illumina 1.3+ FASTQ conversion (PRIVATE).
 
     Avoids creating SeqRecord and Seq objects in order to speed up this
@@ -2206,7 +2212,7 @@ def _fastq_solexa_convert_fastq_illumina(in_file, out_file):
     """
     # Map unexpected chars to null
     mapping = "".join(
-        [chr(0) for ascii in range(0, 59)]
+        [chr(0) for ascii in range(59)]
         + [
             chr(64 + int(round(phred_quality_from_solexa(q))))
             for q in range(-5, 62 + 1)
@@ -2217,7 +2223,9 @@ def _fastq_solexa_convert_fastq_illumina(in_file, out_file):
     return _fastq_generic(in_file, out_file, mapping)
 
 
-def _fastq_illumina_convert_fastq_solexa(in_file, out_file):
+def _fastq_illumina_convert_fastq_solexa(
+    in_file: _TextIOSource, out_file: _TextIOSource
+) -> int:
     """Fast Illumina 1.3+ FASTQ to Solexa FASTQ conversion (PRIVATE).
 
     Avoids creating SeqRecord and Seq objects in order to speed up this
@@ -2225,15 +2233,15 @@ def _fastq_illumina_convert_fastq_solexa(in_file, out_file):
     """
     # Map unexpected chars to null
     mapping = "".join(
-        [chr(0) for ascii in range(0, 64)]
-        + [chr(64 + int(round(solexa_quality_from_phred(q)))) for q in range(0, 62 + 1)]
+        [chr(0) for ascii in range(64)]
+        + [chr(64 + int(round(solexa_quality_from_phred(q)))) for q in range(62 + 1)]
         + [chr(0) for ascii in range(127, 256)]
     )
     assert len(mapping) == 256
     return _fastq_generic(in_file, out_file, mapping)
 
 
-def _fastq_convert_fasta(in_file, out_file):
+def _fastq_convert_fasta(in_file: _TextIOSource, out_file: _TextIOSource) -> int:
     """Fast FASTQ to FASTA conversion (PRIVATE).
 
     Avoids dealing with the FASTQ quality encoding, and creating SeqRecord and
@@ -2254,7 +2262,7 @@ def _fastq_convert_fasta(in_file, out_file):
     return count
 
 
-def _fastq_convert_tab(in_file, out_file):
+def _fastq_convert_tab(in_file: _TextIOSource, out_file: _TextIOSource) -> int:
     """Fast FASTQ to simple tabbed conversion (PRIVATE).
 
     Avoids dealing with the FASTQ quality encoding, and creating SeqRecord and
@@ -2272,7 +2280,11 @@ def _fastq_convert_tab(in_file, out_file):
     return count
 
 
-def _fastq_convert_qual(in_file, out_file, mapping):
+def _fastq_convert_qual(
+    in_file: _TextIOSource,
+    out_file: _TextIOSource,
+    mapping: Mapping[str, str],
+) -> int:
     """FASTQ helper function for QUAL output (PRIVATE).
 
     Mapping should be a dictionary mapping expected ASCII characters from the
@@ -2286,7 +2298,7 @@ def _fastq_convert_qual(in_file, out_file, mapping):
             out_handle.write(f">{title}\n")
             # map the qual... note even with Sanger encoding max 2 digits
             try:
-                qualities_strs = [mapping[ascii] for ascii in qual]
+                qualities_strs = [mapping[ascii_] for ascii_ in qual]
             except KeyError:
                 raise ValueError("Invalid character in quality string") from None
             data = " ".join(qualities_strs)
@@ -2307,13 +2319,13 @@ def _fastq_convert_qual(in_file, out_file, mapping):
     return count
 
 
-def _fastq_sanger_convert_qual(in_file, out_file):
+def _fastq_sanger_convert_qual(in_file: _TextIOSource, out_file: _TextIOSource) -> int:
     """Fast Sanger FASTQ to QUAL conversion (PRIVATE)."""
-    mapping = {chr(q + 33): str(q) for q in range(0, 93 + 1)}
+    mapping = {chr(q + 33): str(q) for q in range(93 + 1)}
     return _fastq_convert_qual(in_file, out_file, mapping)
 
 
-def _fastq_solexa_convert_qual(in_file, out_file):
+def _fastq_solexa_convert_qual(in_file: _TextIOSource, out_file: _TextIOSource) -> int:
     """Fast Solexa FASTQ to QUAL conversion (PRIVATE)."""
     mapping = {
         chr(q + 64): str(int(round(phred_quality_from_solexa(q))))
@@ -2322,9 +2334,11 @@ def _fastq_solexa_convert_qual(in_file, out_file):
     return _fastq_convert_qual(in_file, out_file, mapping)
 
 
-def _fastq_illumina_convert_qual(in_file, out_file):
+def _fastq_illumina_convert_qual(
+    in_file: _TextIOSource, out_file: _TextIOSource
+) -> int:
     """Fast Illumina 1.3+ FASTQ to QUAL conversion (PRIVATE)."""
-    mapping = {chr(q + 64): str(q) for q in range(0, 62 + 1)}
+    mapping = {chr(q + 64): str(q) for q in range(62 + 1)}
     return _fastq_convert_qual(in_file, out_file, mapping)
 
 
