@@ -360,7 +360,7 @@ blocks_converter(PyObject* object, void* pointer)
     }
     if (view->shape[1] != 2) {
         PyErr_Format(PyExc_RuntimeError,
-                     "blocks should have two colums (found %zd)",
+                     "blocks should have two columns (found %zd)",
                      view->shape[1]);
         goto exit;
     }
@@ -457,7 +457,7 @@ static struct PyMethodDef _twoBitIO_methods[] = {
      METH_VARARGS | METH_KEYWORDS,
      TwoBit_convert__doc__
     },
-    {NULL,          NULL, 0, NULL} /* sentinel */
+    {NULL, NULL, 0, NULL} /* sentinel */
 };
 
 
